/*
 * Copyright (c) 2014-2017 Satlab Aps <satlab@satlab.com>
 * All rights reserved. Do not distribute without permission.
 */

#include <util/trace.h>

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <util/util.h>
#include <util/colorprint.h>
#include <util/timer.h>
#include <slash/slash.h>

TRACE_GROUP(tg_user, "user");

slash_command_group(trace, "Trace subcommands");

static int cmd_trace_add(struct slash *slash)
{
	int c, level;
	const char *levelstr = "info";

	if (slash->argc < 2)
		return SLASH_EUSAGE;

	while ((c = slash_getopt(slash, "l:")) != -1) {
		switch (c) {
		case 'l':
			levelstr = slash->optarg;
			break;
		default:
			return SLASH_EUSAGE;
		}
	}

	level = trace_level_from_string(levelstr);
	if (level < 0 || level >= __TRACE_LEVEL_MAX) {
		printf("Invalid level: %s\n", levelstr);
		return SLASH_EINVAL;
	}
	
	if (slash->optind >= slash->argc)
		return SLASH_EUSAGE;

	trace_message(level, tg_user, "%s\n", slash->argv[slash->optind]);

	return 0;
}
slash_command_sub(trace, add, cmd_trace_add, "[-l level] <message>",
		  "Add message to trace log");

static int cmd_trace_print(struct slash *slash)
{
	struct trace_group *group;
	int ret;

	if (slash->argc != 3)
		return SLASH_EUSAGE;

	group = trace_group_find(slash->argv[1]);
	if (!group) {
		printf("Unknown group %s\n", slash->argv[1]);
		return SLASH_EUSAGE;
	}

	ret = trace_mask_from_string(slash->argv[2], &group->print_mask);
	if (ret < 0) {
		printf("Unknown level %s\n", slash->argv[2]);
		return SLASH_EUSAGE;
	}

	return 0;
}
slash_command_sub(trace, print, cmd_trace_print, "<group> <level>",
		  "Set trace group print mask");

static int cmd_trace_store(struct slash *slash)
{
	struct trace_group *group;
	int ret;

	if (slash->argc != 3)
		return SLASH_EUSAGE;

	group = trace_group_find(slash->argv[1]);
	if (!group)
		return SLASH_EUSAGE;

	ret = trace_mask_from_string(slash->argv[2], &group->print_mask);
	if (ret < 0)
		return SLASH_EUSAGE;

	return 0;
}
slash_command_sub(trace, store, cmd_trace_store, "<group> <level>",
		  "Set trace group store mask");

static void trace_group_print(struct trace_group *group)
{
	printf("%-16s%c%c%c%c  %c%c%c%c\n",
	       group->name,
	       (group->print_mask & TRACE_MASK_ERROR) ? 'E' : '.',
	       (group->print_mask & TRACE_MASK_WARN)  ? 'W' : '.',
	       (group->print_mask & TRACE_MASK_INFO)  ? 'I' : '.',
	       (group->print_mask & TRACE_MASK_DEBUG) ? 'D' : '.',
	       (group->store_mask & TRACE_MASK_ERROR) ? 'E' : '.',
	       (group->store_mask & TRACE_MASK_WARN)  ? 'W' : '.',
	       (group->store_mask & TRACE_MASK_INFO)  ? 'I' : '.',
	       (group->store_mask & TRACE_MASK_DEBUG) ? 'D' : '.');
}

static int cmd_trace_list(struct slash *slash)
{
	cprintf(COLOR_BOLD, "Group           Print Store\n");
	trace_groups_iterate(trace_group_print);

	return 0;
}
slash_command_sub(trace, list, cmd_trace_list, NULL, "List available trace groups");
