/*
 * Copyright (c) 2016-2017 Satlab Aps <satlab@satlab.com>
 * All rights reserved. Do not distribute without permission.
 */

#include <stdarg.h>

#include <util/trace.h>

#include <csp/csp.h>
#include <csp/csp_debug.h>

TRACE_GROUP_MASKED(tg_csp, "csp", 0);
#define TRACE_DEFAULT tg_csp

void sl_csp_debug_hook(csp_debug_level_t level, const char *format, va_list args)
{
	trace_level_t tr_level;

	switch (level) {
	case CSP_ERROR:
		tr_level = TRACE_LEVEL_ERROR;
		break;
	case CSP_WARN:
		tr_level = TRACE_LEVEL_WARN;
		break;
	case CSP_INFO:
		tr_level = TRACE_LEVEL_INFO;
		break;
	case CSP_PACKET:
	case CSP_PROTOCOL:
		tr_level = TRACE_LEVEL_DEBUG;
		break;
	case CSP_LOCK:
	case CSP_BUFFER:
	default:
		/* Way to verbose and not really useful */
		return;
	}

	trace_message_va(tr_level, tg_csp, format, args);
}

int sl_csp_debug_hook_enable(void)
{
	/* Enable all log output from CSP */
	csp_debug_set_level(CSP_ERROR,    true);
	csp_debug_set_level(CSP_WARN,     true);
	csp_debug_set_level(CSP_INFO,     true);
	csp_debug_set_level(CSP_PACKET,   true);
	csp_debug_set_level(CSP_PROTOCOL, true);
	csp_debug_set_level(CSP_LOCK,     false);
	csp_debug_set_level(CSP_BUFFER,   false);

	/* Redirect to our debug hook */
	csp_debug_hook_set(sl_csp_debug_hook);

	return 0;
}
